use std::default;

use bevy::{
	asset::{embedded_asset, embedded_path},
	diagnostic::FrameTimeDiagnosticsPlugin,
	prelude::*,
	render::camera::Viewport,
	window::{WindowMode, WindowResolution},
};
use common::mouse::{Mouse, MousePlugin};
use hud::HUDPlugin;
use level::LevelPlugin;
use player::PlayerPlugin;
use score::ScorePlugin;

mod common;
mod enemy;
mod hud;
mod level;
mod player;
mod score;

pub const SCREEN_WIDTH: u32 = 480;
pub const SCREEN_HEIGHT: u32 = 720;
pub const START_POSITION: Vec3 =
	Vec3::new((SCREEN_WIDTH / 2) as f32, (SCREEN_HEIGHT / 2) as f32, 0.0);

#[derive(Clone, Eq, PartialEq, Debug, Hash, Default, States)]
pub enum GameState {
	#[default]
	Menu,
	Playing,
	GameOver,
}

#[derive(Event, Default)]
struct GameOverEvent;

#[derive(Event, Default)]
struct GameStartEvent;

fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
	commands.spawn(Camera2dBundle {
		camera: Camera {
			viewport: Some(Viewport {
				physical_position: UVec2::new(0, 0),
				physical_size: UVec2::new(480, 720),
				..Default::default()
			}),
			..Default::default()
		},
		transform: Transform {
			translation: Vec3 {
				x: (SCREEN_WIDTH / 2) as f32,
				y: (SCREEN_HEIGHT / 2) as f32,
				z: 1.0,
			},
			..Default::default()
		},
		..Default::default()
	});
}

fn debug_mouse(mouse: Res<Mouse>) {
	// println!("x: {} y: {}", mouse.x, mouse.y);
}

fn main() {
	App::new()
		.add_plugins((
			DefaultPlugins
				.build()
				.set(WindowPlugin {
					primary_window: Some(Window {
						title: "Avoid".to_string(),
						name: Some("avoid.app".to_string()),
						resizable: false,
						position: WindowPosition::Centered(MonitorSelection::Current),
						resolution: WindowResolution::new(
							SCREEN_WIDTH as f32,
							SCREEN_HEIGHT as f32,
						),
						..Default::default()
					}),
					..Default::default()
				})
				.set(ImagePlugin::default_nearest()),
			FrameTimeDiagnosticsPlugin,
			// EmbeddedAssetPlugin,
			LevelPlugin,
			MousePlugin,
		))
		.init_state::<GameState>()
		.add_event::<GameOverEvent>()
		.add_event::<GameStartEvent>()
		.add_systems(Startup, setup)
		.add_systems(Update, debug_mouse)
		.run();
}

struct EmbeddedAssetPlugin;

// impl Plugin for EmbeddedAssetPlugin {
// 	fn build(&self, app: &mut App) {
// 		let prefix = "avoid-rs/";
//
// 		// embedded_asset!(app, "../assets/player.png");
// 		embedded_asset!(app, "../assets/Xolonium-Regular.ttf");
// 		// embedded_asset!(app, "./assets/Xolonium-Regular.ttf");
// 	}
// }
